#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"stringid.h"


/*
	Key set.
*/


// Macro.


// Initial size of hash table (1 << HASH_MIN).
#define		HASH_MIN	8


// Structures.


// File for key set.
typedef		struct SIFile
{
	int		size;	// File size.
	int		pos;	// Position.
	char	buf[];	// Buffer.
}	SIFile, *hSIFile;

// Key.
typedef		struct SIKey	*hSIKey;
typedef		struct SIKey
{
	hSIKey	next;	// Next key.
	char	s[];	// Key.
}	SIKey;

// Key set.
typedef		struct SISet
{
	hSIKey	*tbl;	// Hash table.
	int		nbit;	// Size of hash table (size = 1 << nbit).
	int		mask;	// Mask of hash value (mask = size - 1).
	int		n;		// Number of keys.
}	SISet;

// Sorted key set.
typedef		struct SISort
{
	int		n;		// Number of keys.
	int		max;	// Maximum length of keys.
	int		total;	// Total length of keys.
	hSIKey	key[];	// Keys.
}	SISort, *hSISort;


// Strings.


// Get length.
static	int		SIStr_Len( const char *s )
{
	const char	*p = s;

	while ( *p != '\0' )
		p++;

	return	( int )( p - s );
}

// Copy.
static	void	SIStr_Copy( char *dst, const char *src )
{
	do
		*dst++ = *src;
	while ( *src++ != '\0' );
}

// Compare.
static	int		SIStr_Cmp( const char *a, const char *b )
{
	while ( *a != '\0' && *a == *b )
		a++, b++;

	return	( unsigned char )*a - ( unsigned char )*b;
}

// Exact match.
static	const char	*SIStr_Match( const char *a, const char *b, int *n )
{
	while ( a[ *n ] != '\0' && a[ *n ] == b[ *n ] )
		++*n;

	return	( a[ *n ] == b[ *n ] ) ? a : NULL;
}

// Prefix match.
static	const char	*SIStr_Prefix( const char *a, const char *b, int *n )
{
	if ( a[ *n ] != b[ *n ] )
		return	NULL;

	while ( a[ *n ] != '\0' && a[ *n ] == b[ *n ] )
		++*n;

	return	( a[ *n ] == '\0' ) ? a : NULL;
}


// Initialize and finish key.


// Initialize.
static	void	SIKey_Init( hSIKey h, const char *key )
{
	memset( h, 0, sizeof( *h ) );

	h->next = NULL;

	SIStr_Copy( h->s, key );
}

// Create.
static	hSIKey	SIKey_Create( const char *key )
{
	int		len;
	hSIKey	h;

	len = SIStr_Len( key );
	if ( ( h = ( hSIKey )malloc( sizeof( *h ) + len + 1 ) ) != NULL )
		SIKey_Init( h, key );

	return	h;
}

// Finish.
static	void	SIKey_Finish( hSIKey h )
{
}

// Destroy.
static	void	SIKey_Delete( hSIKey h )
{
	SIKey_Finish( h );
	free( h );
}


// Initialize and finish key set.


// Initialize.
static	void	SISet_Init( hSISet h )
{
	memset( h, 0, sizeof( *h ) );
}

// Create.
static	hSISet	SISet_Create( void )
{
	hSISet	h;

	if ( ( h = ( hSISet )malloc( sizeof( *h ) ) ) != NULL )
		SISet_Init( h );

	return	h;
}

// Finish.
static	void	SISet_Finish( hSISet h )
{
	int		i, max;
	hSIKey	key, next;

	if ( h->tbl != NULL )
	{
		for ( i = 0, max = 1 << h->nbit; i < max; i++ )
		{
			for ( key = h->tbl[i]; key != NULL; key = next )
			{
				next = key->next;
				SIKey_Delete( key );
			}
		}
		free( h->tbl );
	}
}

// Destroy.
static	void	SISet_Delete( hSISet h )
{
	SISet_Finish( h );
	free( h );
}


// Hash.


// Calculate hash value.
static	int		SISet_Hash( const char *key )
{
	unsigned	val = 0;

	while ( *key != '\0' )
		val += ( val << 5 ) + ( unsigned char )*key++;

	return	( int )( val >> 1 );
}

// Move keys into expanded hash table.
static	void	SISet_Divide( hSISet h, int nbit )
{
	int		i, max, mask, val;
	hSIKey	*p, key;

	max = 1 << h->nbit;
	mask = ( 1 << nbit ) - 1;

	for ( i = 0; i < max; i++ )
	{
		// Initialize new area.
		h->tbl[ max + i ] = NULL;
		for ( p = &h->tbl[i]; *p != NULL; )
		{
			// Calculate hash value.
			if ( ( val = SISet_Hash( ( *p )->s ) & mask ) != i )
			{
				key = *p;
				*p = ( *p )->next;
				key->next = h->tbl[val];
				h->tbl[val] = key;
			}
			else
				p = &( *p )->next;
		}
	}
}

// Expand hash table.
static	int		SISet_Expand( hSISet h )
{
	int		i, max, nbit;
	hSIKey	*tbl;

	// Calculate size and allocate memory.
	nbit = ( h->nbit < HASH_MIN ) ? HASH_MIN : ( h->nbit + 1 );
	tbl = ( hSIKey * )realloc( h->tbl, sizeof( *tbl ) << nbit );
	if ( tbl == NULL )
		return	SI_EMEM;
	h->tbl = tbl;

	// Initialize hash table or move keys.
	if ( h->nbit < HASH_MIN )
	{
		for ( i = 0, max = 1 << nbit; i < max; i++ )
			tbl[i] = NULL;
	}
	else
		SISet_Divide( h, nbit );

	h->nbit = nbit;
	h->mask = ( 1 << nbit ) - 1;

	return	SI_OK;
}

// Search key.
static	hSIKey	*SISet_Search( hSISet h, const char *key )
{
	int		val;
	hSIKey	*p;

	val = SISet_Hash( key ) & h->mask;
	for ( p = &h->tbl[val]; *p != NULL; p = &( *p )->next )
	{
		if ( !SIStr_Cmp( ( *p )->s, key ) )
			break;
	}

	return	p;
}

// Add key to hash table.
static	int		SISet_Add( hSISet h, const char *key )
{
	int		ret;
	hSIKey	*p;

	// Expand hash table.
	if ( h->n >= 1 << h->nbit )
	{
		if ( ( ret = SISet_Expand( h ) ) != SI_OK )
			return	ret;
	}

	// Create new key, if the received key is not registered.
	if ( *( p = SISet_Search( h, key ) ) == NULL )
	{
		if ( ( *p = SIKey_Create( key ) ) == NULL )
			return	SI_EMEM;
		h->n++;
	}

	return	SI_OK;
}


// Key file.


// Initialize.
static	void	SIFile_Init( hSIFile h, int size )
{
	memset( h, 0, sizeof( *h ) );

	h->size = size;
}

// Create.
static	hSIFile		SIFile_Create( int size )
{
	hSIFile		h;

	if ( ( h = ( hSIFile )malloc( sizeof( *h ) + size + 1 ) ) != NULL )
		SIFile_Init( h, size );

	return	h;
}

// Finish.
static	void	SIFile_Finish( hSIFile h )
{
}

// Destroy.
static	void	SIFile_Delete( hSIFile h )
{
	SIFile_Finish( h );
	free( h );
}

// Load data.
static	int		SIFile_Load( hSIFile h, FILE *fp )
{
	if ( fread( h->buf, h->size, 1, fp ) != 1 )
		return	SI_EIO;
	h->buf[ h->size ] = '\0';

	return	SI_OK;
}

// Allocate memory.
static	int		SIFile_Alloc( hSIFile *h, FILE *fp )
{
	int		size;

	// File size.
	if ( fseek( fp, 0, SEEK_END ) == -1 ||
		 ( size = ftell( fp ) ) == -1 ||
		 fseek( fp, 0, SEEK_SET ) == -1 )
		return	SI_EIO;

	if ( ( *h = SIFile_Create( size ) ) == NULL )
		return	SI_EMEM;

	return	SI_OK;
}

// Load file.
static	int		SIFile_Open( hSIFile *h, const char *path )
{
	int		ret;
	FILE	*fp;

	if ( ( fp = fopen( path, "rb" ) ) == NULL )
		return	SI_EIO;

	if ( ( ret = SIFile_Alloc( h, fp ) ) == SI_OK )
	{
		if ( ( ret = SIFile_Load( *h, fp ) ) != SI_OK )
			SIFile_Delete( *h );
	}

	fclose( fp );

	return	ret;
}

// Free memory.
static	void	SIFile_Close( hSIFile h )
{
	SIFile_Delete( h );
}

// Check delimitor or not.
static	int		SIFile_IsDelim( const char *delim, char c )
{
	while ( *delim != '\0' )
	{
		if ( c == *delim++ )
			return	1;
	}

	return	c == '\0';
}

// Extract key.
static	const char	*SIFile_Delim( hSIFile h, const char *delim )
{
	const char	*start;

	// Find start position.
	while ( h->pos < h->size && SIFile_IsDelim( delim, h->buf[ h->pos ] ) )
		h->pos++;
	if ( h->pos >= h->size )
		return	NULL;
	start = h->buf + h->pos;

	// Find end of key.
	while ( !SIFile_IsDelim( delim, h->buf[ h->pos ] ) )
		h->pos++;
	h->buf[ h->pos++ ] = '\0';

	return	start;
}

// Add keys listed in file.
static	int		SISet_FileAdd( hSISet h, hSIFile file, const char *delim )
{
	int		ret;
	const char	*key;

	while ( ( key = SIFile_Delim( file, delim ) ) != NULL )
	{
		if ( ( ret = SISet_Insert( h, key ) ) != SI_OK )
			return	ret;
	}

	return	SI_OK;
}


// Initialize and finish sorting.


// Iniaialize.
static	void	SISort_Init( hSISort h, int n )
{
	memset( h, 0, sizeof( *h ) );

	h->n = n;
}

// Create.
static	hSISort		SISort_Create( int n )
{
	int		size;
	hSISort		h;

	size = sizeof( *h ) + sizeof( *h->key ) * n;
	if ( ( h = ( hSISort )malloc( size ) ) != NULL )
		SISort_Init( h, n );

	return	h;
}

// Finish.
static	void	SISort_Finish( hSISort h )
{
}

// Destroy
static	void	SISort_Delete( hSISort h )
{
	SISort_Finish( h );
	free( h );
}


// Sort.


// Multi key quicksort.
static	void	SISort_Quick( hSIKey *key, int n, int depth );


// Swap keys.
static	void	SISort_Swap( hSIKey *key, int x, int y )
{
	hSIKey	tmp;

	tmp = key[x];
	key[x] = key[y];
	key[y] = tmp;
}

// Select pivot.
static	void	SISort_Pivot( hSIKey *key, int n, int depth )
{
	// Left is smaller than center.
	if ( ( unsigned char )key[0]->s[depth] <
		 ( unsigned char )key[ n >> 1 ]->s[depth] )
		SISort_Swap( key, 0, n >> 1 );

	// Right is smaller than center.
	if ( ( unsigned char )key[ n - 1 ]->s[depth] <
		 ( unsigned char )key[ n >> 1 ]->s[depth] )
	{
		SISort_Swap( key, 0, n >> 1 );
		SISort_Swap( key, n >> 1, n - 1 );
	}
	// Right is smaller than left.
	else if ( ( unsigned char )key[ n - 1 ]->s[depth] <
			  ( unsigned char )key[0]->s[depth] )
		SISort_Swap( key, 0, n - 1 );

	// Swap next to left and center.
	SISort_Swap( key, 1, n >> 1 );
}

// Insert sort.
static	void	SISort_Insert( hSIKey *key, int n, int depth )
{
	int		i, j;
	const char	*s;

	for ( i = 1; i < n; i++ )
	{
		s = key[i]->s + depth;
		for ( j = i; j > 0; j-- )
		{
			if ( SIStr_Cmp( s, key[ j - 1 ]->s + depth ) > 0 )
				break;
			SISort_Swap( key, j - 1, j );
		}
	}
}

// Subroutine of multi key quicksort.
static	void	SISort_QuickSub( hSIKey *key, int n, int depth,
								 int l, unsigned char pivot )
{
	int		i, r, pl, pr, mv, diff;

	// Initialize range.
	pl = l;
	pr = r = n - 1;

	// Partitioning.
	for ( ; ; )
	{
		// Move pivots to left.
		while ( l <= r )
		{
			if ( ( diff = ( unsigned char )key[l]->s[depth] - pivot ) > 0 )
				break;
			else if ( !diff )
				SISort_Swap( key, l, pl++ );
			l++;
		}

		// Move pivots to right.
		while ( l <= r )
		{
			if ( ( diff = ( unsigned char )key[r]->s[depth] - pivot ) < 0 )
				break;
			else if ( !diff )
				SISort_Swap( key, r, pr-- );
			r--;
		}

		if ( l > r )
			break;

		SISort_Swap( key, l++, r-- );
	}

	// Move left pivots to center.
	mv = ( pl < l - pl ) ? pl : l - pl;
	for ( i = 0; i < mv; i++ )
		SISort_Swap( key, i, r - i );
	pl = l - pl;

	// Move right pivots to center.
	n--;
	mv = ( n - pr < pr - r ) ? n - pr : pr - r;
	for ( i = 0; i < mv; i++ )
		SISort_Swap( key, l + i, n - i );
	pr = ++n - ( pr - r );

	// Sort recursively.
	if ( pl > 1 )
		SISort_Quick( key, pl, depth );
	if ( pr - pl > 1 )
		SISort_Quick( key + pl, pr - pl, depth + 1 );
	if ( n - pr > 1 )
		SISort_Quick( key + pr, n - pr, depth );
}

// Multi key quicksort.
static	void	SISort_Quick( hSIKey *key, int n, int depth )
{
	int		i;
	unsigned char	pivot;

	if ( n < 8 )
	{
		// Use insert sort, if there are few elements.
		SISort_Insert( key, n, depth );
		return;
	}

	depth--;
	do
	{
		// Move pivots to left.
		SISort_Pivot( key, n, ++depth );
		pivot = ( unsigned char )key[0]->s[depth];

		// Check range.
		for ( i = 1; i < n; i++ )
		{
			if ( ( unsigned char )key[i]->s[depth] != pivot )
				break;
		}
	}	while ( i == n );

	SISort_QuickSub( key, n, depth, i, pivot );
}

// Bin sort.
static	void	SISort_Bin( hSIKey *key, hSISet set )
{
	int		i, max, depth;
	int		buf[258], *freq;
	hSIKey	p;

	// Initialize.
	depth = 0;
	max = 1 << set->nbit;

	do
	{
		// Calculate frequencies of characters in depth.
		freq = buf + 2;
		memset( buf, 0, sizeof( buf ) );
		for ( i = 0; i < max; i++ )
		{
			for ( p = set->tbl[i]; p != NULL; p = p->next )
				freq[ ( unsigned char )p->s[depth] ]++;
		}

		// Total frequencies.
		for ( i = 1, freq--; i < 256; i++ )
		{
			if ( freq[i] == set->n && set->n > 1 )
			{
				depth++;
				break;
			}
			freq[i] += freq[ i - 1 ];
		}
	}	while ( i < 256 );

	// Divide.
	for ( i = 0; i < max; i++ )
	{
		for ( p = set->tbl[i]; p != NULL; p = p->next )
			key[ freq[ ( unsigned char )p->s[depth] ]++ ] = p;
	}

	// Multi key quicksort.
	for ( i = 0, freq--, depth++; i < 256; i++ )
	{
		if ( freq[ i + 1 ] - freq[i] > 1 )
			SISort_Quick( key + freq[i], freq[ i + 1 ] - freq[i], depth );
	}
}

// Check length of keys.
static	void	SISort_Check( hSISort h )
{
	int		i, len;

	// Get length of the longest key.
	for ( i = 0; i < h->n; i++ )
	{
		len = SIStr_Len( h->key[i]->s );
		h->total += len;
		if ( len > h->max )
			h->max = len;
	}
}

// Sort key set.
static	int		SISort_Open( hSISort *h, hSISet set )
{
	if ( ( *h = SISort_Create( set->n ) ) == NULL )
		return	SI_EMEM;

	SISort_Bin( ( *h )->key, set );

	SISort_Check( *h );

	return	SI_OK;
}

// Destroy sorted key set.
static	void	SISort_Close( hSISort h )
{
	SISort_Delete( h );
}


// Interfaces.


// Create key set.
int		SISet_Open( hSISet *h )
{
	int		ret;

	if ( h == NULL )
		return	SI_EHDL;

	if ( ( *h = SISet_Create() ) == NULL )
		return	SI_EMEM;

	if ( ( ret = SISet_Expand( *h ) ) != SI_OK )
	{
		SISet_Delete( *h );
		*h = NULL;
	}

	return	ret;
}

// Load key set.
int		SISet_Load( hSISet h, const char *path, const char *delim )
{
	int		ret;
	hSIFile		file;

	if ( h == NULL )
		return	SI_EHDL;
	else if ( path == NULL || delim == NULL )
		return	SI_EARG;

	if ( ( ret = SIFile_Open( &file, path ) ) != SI_OK )
		return	ret;

	ret = SISet_FileAdd( h, file, delim );

	SIFile_Close( file );

	return	SI_OK;
}

// Insert key.
int		SISet_Insert( hSISet h, const char *key )
{
	if ( h == NULL )
		return	SI_EHDL;
	else if ( key == NULL )
		return	SI_EARG;

	return	SISet_Add( h, key );
}

// Destroy key set.
int		SISet_Close( hSISet h )
{
	if ( h == NULL )
		return	SI_EHDL;

	SISet_Delete( h );

	return	SI_OK;
}


/*
	Double-array dictionary.
*/


// Macros.


// Version.
#define		SI_VERSION	121		// Current.
#define		SI_COMPAT	120		// Compatible.

// Identifier.
#define		HEAD_STR	"SID"

// Temporary flags for construction.
#define		FLAG_USED	( 1 << 0 )	// This element is used or not.
#define		FLAG_BASE	( 1 << 1 )	// This base is used or not.

// Upper limits of size.
#define		BC_MAX		( VAL_BASE + 1 )	// 8,388,608
#define		TAIL_MAX	VAL_LINK			// 2,147,483,647

// Index of root node.
#define		IDX_ROOT	0

// BC.
#define		VAL_LEAF	0x00000001U
#define		VAL_BASE	0x007FFFFFU
#define		VAL_CHECK	0x000000FFU
#define		VAL_LINK	0x7FFFFFFFU
#define		OFF_LEAF	31
#define		OFF_BASE	8
#define		OFF_CHECK	0
#define		OFF_LINK	0
#define		MASK_LEAF	( VAL_LEAF << OFF_LEAF )
#define		MASK_BASE	( VAL_BASE << OFF_BASE )
#define		MASK_CHECK	( VAL_CHECK << OFF_CHECK )
#define		MASK_LINK	( VAL_LINK << OFF_LINK )


// Structures.


// Element of BC.
typedef		unsigned	SIInt, *hSIInt;

// Arc.
typedef		struct SIArc
{
	int		c;	// Arc label.
	int		n;	// Frequency.
}	SIArc, *hSIArc;

// Siblings.
typedef		struct SISib
{
	SIArc	arc[256];	// Arcs.
	int		n;			// Number of arcs.
}	SISib, *hSISib;

// Temporary information for construction.
typedef		struct SIInf
{
	hSIDic		dic;	// Dictionary.
	hSISort		sort;	// Sorted key set.
	hSISib		sib;	// Siblings.
	char		*flag;	// USED and BASE flags.
	int			n;		// Length of flag array.
	int			ntail;	// Length of TAIL.
	int			max;	// Rearmost used element.
	int			empty;	// First empty element.
	int			pos;	// Next position of TAIL.
}	SIInf, *hSIInf;

// Dictionary header.
typedef		struct SIHead
{
	char	str[4];		// Identifer.
	int		version;	// Version.
	int		nbc;		// Length of BC.
	int		ntail;		// Length of TAIL.
	int		nkey;		// Number of keys.
	int		maxlen;		// Length of the longest key.
	int		space;		// Reserved.
}	SIHead, *hSIHead;

// Dictionary.
typedef		struct SIDic
{
	hSIInt	bc;			// Array BC.
	char	*tail;		// Array TAIL.
	int		nbc;		// Length of BC.
	int		ntail;		// Length of TAIL.
	int		nkey;		// Number of keys.
	int		maxlen;		// Length of the longest key.
}	SIDic;


// Element access.


// Read LEAF.
static	int		SIInt_Leaf( hSIInt h )
{
	return	( int )( ( *h & MASK_LEAF ) >> OFF_LEAF );
}

// Write LEAF.
static	void	SIInt_WLeaf( hSIInt h, int val )
{
	*h &= ~MASK_LEAF;
	*h |= ( ( SIInt )val & VAL_LEAF ) << OFF_LEAF;
}

// Read BASE.
static	int		SIInt_Base( hSIInt h )
{
	return	( int )( ( *h & MASK_BASE ) >> OFF_BASE );
}

// Write BASE.
static	void	SIInt_WBase( hSIInt h, int val )
{
	*h &= ~MASK_BASE;
	*h |= ( ( SIInt )val & VAL_BASE ) << OFF_BASE;
}

// Read CHECK.
static	int		SIInt_Check( hSIInt h )
{
	return	( int )( ( *h & MASK_CHECK ) >> OFF_CHECK );
}

// Write CHECK.
static	void	SIInt_WCheck( hSIInt h, int val )
{
	*h &= ~MASK_CHECK;
	*h |= ( ( SIInt )val & VAL_CHECK ) << OFF_CHECK;
}

// Read LINK.
static	int		SIInt_Link( hSIInt h )
{
	return	( int )( ( *h & MASK_LINK ) >> OFF_LINK );
}

// Write LINK.
static	void	SIInt_WLink( hSIInt h, int val )
{
	*h &= ~MASK_LINK;
	*h |= ( ( SIInt )val & VAL_LINK ) << OFF_LINK;
}


// Initialize and finish dictionary.


// Initialize.
static	void	SIDic_Init( hSIDic h )
{
	memset( h, 0, sizeof( *h ) );

	h->bc = NULL;
	h->tail = NULL;
}

// Create.
static	hSIDic	SIDic_Create( void )
{
	hSIDic	h;

	if ( ( h = ( hSIDic )malloc( sizeof( *h ) ) ) != NULL )
		SIDic_Init( h );

	return	h;
}

// Finish.
static	void	SIDic_Finish( hSIDic h )
{
	if ( h->bc != NULL )
		free( h->bc );
	if ( h->tail != NULL )
		free( h->tail );
}

// Destroy.
static	void	SIDic_Delete( hSIDic h )
{
	SIDic_Finish( h );
	free( h );
}

// Resize BC.
static	int		SIDic_ResizeBC( hSIDic h, int n )
{
	hSIInt	bc;

	if ( ( bc = ( hSIInt )realloc( h->bc, sizeof( *bc ) * n ) ) == NULL )
		return	SI_EMEM;

	h->bc = bc;
	h->nbc = n;

	return	SI_OK;
}

// Resize TAIL.
static	int		SIDic_ResizeTail( hSIDic h, int n )
{
	char	*tail;

	if ( ( tail = ( char * )realloc( h->tail, n ) ) == NULL )
		return	SI_EMEM;

	h->tail = tail;
	h->ntail = n;

	return	SI_OK;
}


// Dictionary access.


// Read LEAF.
static	int		SIDic_Leaf( hSIDic h, int idx )
{
	return	SIInt_Leaf( h->bc + idx );
}

// Write LEAF.
static	void	SIDic_WLeaf( hSIDic h, int idx, int val )
{
	SIInt_WLeaf( h->bc + idx, val );
}

// Read BASE.
static	int		SIDic_Base( hSIDic h, int idx )
{
	return	SIInt_Base( h->bc + idx );
}

// Write BASE.
static	void	SIDic_WBase( hSIDic h, int idx, int val )
{
	SIInt_WBase( h->bc + idx, val );
}

// Read CHECK.
static	int		SIDic_Check( hSIDic h, int idx )
{
	return	SIInt_Check( h->bc + idx );
}

// Write CHECK.
static	void	SIDic_WCheck( hSIDic h, int idx, int val )
{
	SIInt_WCheck( h->bc + idx, val );
}

// Read LINK.
static	int		SIDic_Link( hSIDic h, int idx )
{
	return	SIInt_Link( h->bc + idx );
}

// Write LINK.
static	void	SIDic_WLink( hSIDic h, int idx, int val )
{
	SIInt_WLink( h->bc + idx, val );
}

// Read TAIL.
static	char	*SIDic_Str( hSIDic h, int pos )
{
	return	h->tail + pos;
}

// Write TAIL.
static	void	SIDic_WStr( hSIDic h, int pos, const char *s )
{
	SIStr_Copy( h->tail + pos, s );
}


// Initialize and finish temporary.


// Initialize.
static	void	SIInf_Init( hSIInf h, hSIDic dic, hSISort sort )
{
	memset( h, 0, sizeof( *h ) );

	h->dic = dic;
	h->sort = sort;
	h->sib = NULL;
	h->flag = NULL;
}

// Create.
static	hSIInf	SIInf_Create( hSIDic dic, hSISort sort )
{
	hSIInf	h;

	if ( ( h = ( hSIInf )malloc( sizeof( *h ) ) ) != NULL )
		SIInf_Init( h, dic, sort );

	return	h;
}

// Finish.
static	void	SIInf_Finish( hSIInf h )
{
	if ( h->flag != NULL )
		free( h->flag );
	if ( h->sib != NULL )
		free( h->sib );
}

// Destroy.
static	void	SIInf_Delete( hSIInf h )
{
	SIInf_Finish( h );
	free( h );
}


// Siblings.


// Get siblings.
static	void	SISib_Scan( hSISib h, hSIKey *key, int n, int depth )
{
	int		i, type;

	// First arc.
	h->arc[0].c = ( unsigned char )key[0]->s[depth];
	h->arc[0].n = 1;

	for ( i = 1, type = 0; i < n; i++ )
	{
		if ( h->arc[type].c == ( unsigned char )key[i]->s[depth] )
			h->arc[type].n++;
		else
		{
			h->arc[ ++type ].c = ( unsigned char )key[i]->s[depth];
			h->arc[type].n = 1;
		}
	}

	// Number of arcs.
	h->n = type + 1;
}


// Memoery allocation for construction.


// Allocate memory for siblings.
static	int		SIInf_ResizeSib( hSIInf h, int n )
{
	hSISib	sib;

	if ( ( sib = ( hSISib )realloc( h->sib, sizeof( *sib ) * n ) ) == NULL )
		return	SI_EMEM;
	h->sib = sib;

	return	SI_OK;
}

// Allocate memory for siblings.
static	int		SIInf_ExpandSib( hSIInf h )
{
	return	SIInf_ResizeSib( h, h->sort->max + 1 );
}

// Initialize new elements.
static	void	SIInf_ClearBC( hSIInf h, int n )
{
	int		i;
	hSIInt	bc = h->dic->bc;

	if ( n > h->n )
	{
		// Initialize flags.
		memset( h->flag + h->n, 0, n - h->n );

		// Initialize E-LINK.
		for ( i = h->n; i < n; i++ )
			bc[i] = i + 1;
	}

	h->n = n;
}

// Resize BC.
static	int		SIInf_ResizeBC( hSIInf h, int n )
{
	int		ret;
	char	*flag;

	if ( ( flag = ( char * )realloc( h->flag, n ) ) == NULL )
		return	SI_EMEM;
	h->flag = flag;

	if ( ( ret = SIDic_ResizeBC( h->dic, n ) ) != SI_OK )
		return	ret;

	SIInf_ClearBC( h, n );

	return	SI_OK;
}

// Expand BC.
static	int		SIInf_ExpandBC( hSIInf h )
{
	int		size;

	// Upper limit.
	if ( h->n >= BC_MAX )
		return	SI_EMAX;

	// Calculate size.
	size = h->n ? h->n : h->sort->n;
	size = ( ( size + ( size >> 1 ) ) + 0x100 ) & ~0xFF;
	if ( size > BC_MAX )
		size = BC_MAX;

	return	SIInf_ResizeBC( h, size );
}

// Reduce BC.
static	int		SIInf_ReduceBC( hSIInf h )
{
	int		size;

	size = ( h->max + 0x100 ) & ~0xFF;

	return	SIInf_ResizeBC( h, size );
}

// Initialize new elements.
static	void	SIInf_ClearTail( hSIInf h, int n )
{
	if ( n > h->ntail )
		memset( h->dic->tail + h->ntail, 0, n - h->ntail );

	h->ntail = n;
}

// Resize TAIL.
static	int		SIInf_ResizeTail( hSIInf h, int n )
{
	int		ret;

	if ( ( ret = SIDic_ResizeTail( h->dic, n ) ) != SI_OK )
		return	ret;

	SIInf_ClearTail( h, n );

	return	SI_OK;
}

// Reduce TAIL.
static	int		SIInf_ReduceTail( hSIInf h )
{
	if ( h->pos == h->ntail )
		return	SI_OK;

	return	SIDic_ResizeTail( h->dic, h->pos );
}


// Subroutines for construction.


// Used element or not.
static	int		SIInf_Used( hSIInf h, int idx )
{
	return	h->flag[idx] & FLAG_USED;
}

// BASE is used or not.
static	int		SIInf_Base( hSIInf h, int idx )
{
	return	( h->flag[idx] & FLAG_BASE ) == FLAG_BASE;
}

// Next empty element.
static	int		SIInf_NextEmpty( hSIInf h, int idx )
{
	int		next;

	// Candidate.
	next = h->dic->bc[idx];

	// Search.
	while ( next < h->n && SIInf_Used( h, next ) )
		next++;

	// Update candidate.
	h->dic->bc[idx] = next;

	return	next;
}

// Set USED flag.
static	void	SIInf_WUsed( hSIInf h, int idx )
{
	// First empty element.
	if ( idx == h->empty )
		h->empty = SIInf_NextEmpty( h, idx );

	// Rearmost used element.
	if ( idx > h->max )
		h->max = idx;

	h->flag[idx] |= FLAG_USED;
}

// Set BASE flag.
static	void	SIInf_WBase( hSIInf h, int idx )
{
	h->flag[idx] |= FLAG_BASE;
}

// Initialize root.
static	int		SIInf_Root( hSIInf h )
{
	int		ret;

	// Allocate memory (about 1.5 times number of keys).
	if ( ( ret = SIInf_ExpandBC( h ) ) != SI_OK )
		return	ret;

	// Initialize root.
	SIInf_WUsed( h, IDX_ROOT );
	SIInf_WBase( h, IDX_ROOT );
	SIDic_WLeaf( h->dic, IDX_ROOT, 0 );
	SIDic_WBase( h->dic, IDX_ROOT, IDX_ROOT );
	SIDic_WCheck( h->dic, IDX_ROOT, '\0' );

	return	SI_OK;
}


// Build BC.


// Search good offset.
static	int		SIInf_Search( hSIInf h, hSISib sib )
{
	int		i, empty, base;

	// Check only empty elements.
	for ( empty = h->empty; empty < h->n; empty = SIInf_NextEmpty( h, empty ) )
	{
		// Check BASE is used or not.
		base = empty ^ sib->arc[0].c;
		if ( SIInf_Base( h, base ) )
			continue;

		// Check siblings.
		for ( i = 1; i < sib->n; i++ )
		{
			if ( SIInf_Used( h, base ^ sib->arc[i].c ) )
				break;
		}

		// Good offset.
		if ( i == sib->n )
			return	base;
	}

	// First empty element in new block.
	return	empty ^ sib->arc[0].c;
}

// Arrange siblings.
static	int		SIInf_Arrange( hSIInf h, hSISib sib, int idx )
{
	int		i, base, next, ret;

	// Get good offset.
	if ( ( base = SIInf_Search( h, sib ) ) >= h->n )
	{
		if ( ( ret = SIInf_ExpandBC( h ) ) != SI_OK )
			return	ret;
	}

	// Set BASE.
	SIInf_WBase( h, base );
	SIDic_WBase( h->dic, idx, base );

	for ( i = 0; i < sib->n; i++ )
	{
		// Set CHECK.
		next = base ^ sib->arc[i].c;
		SIInf_WUsed( h, next );
		SIDic_WCheck( h->dic, next, sib->arc[i].c );
	}

	return	SI_OK;
}

// Depth first.
static	int		SIInf_Progress( hSIInf h, hSIKey *key, int n,
								int idx, int depth )
{
	int		i, single, base, ret;
	hSISib	sib;

	// Leaf or not.
	single = depth && ( n == 1 );
	SIDic_WLeaf( h->dic, idx, single );
	if ( single )
	{
		SIDic_WLink( h->dic, idx, h->pos );
		SIDic_WStr( h->dic, h->pos, key[0]->s );
		h->pos += SIStr_Len( key[0]->s ) + 1;
		return SI_OK;
	}

	// Get siblings.
	sib = h->sib + depth;
	SISib_Scan( sib, key, n, depth );

	// Arrange.
	if ( ( ret = SIInf_Arrange( h, sib, idx ) ) != SI_OK )
		return	ret;
	base = SIDic_Base( h->dic, idx );

	for ( i = 0; i < sib->n; i++ )
	{
		ret = SIInf_Progress( h, key, sib->arc[i].n,
							  base ^ sib->arc[i].c, depth + 1 );
		if ( ret != SI_OK )
			return	ret;

		key += sib->arc[i].n;
	}

	return	SI_OK;
}

// Build BC.
static	int		SIInf_BuildBC( hSIInf h )
{
	int		ret;

	// Empty dictionary.
	if ( h->sort->n <= 0 )
	{
		SIInf_WBase( h, IDX_ROOT ^ 1 );
		SIDic_WBase( h->dic, IDX_ROOT, IDX_ROOT ^ 1 );
		return	SI_OK;
	}

	if ( ( ret = SIInf_ExpandSib( h ) ) != SI_OK )
		return	ret;

	ret = SIInf_Progress( h, h->sort->key, h->sort->n, IDX_ROOT, 0 );
	if ( ret != SI_OK )
		return	ret;

	return	SI_OK;
}

// Check empty elements.
static	void	SIInf_Revise( hSIInf h )
{
	int		empty, next, base = 0, block = -1;

	for ( empty = h->empty; empty < h->n; empty = next )
	{
		next = SIInf_NextEmpty( h, empty );

		// Search block for unused BASE.
		if ( empty >> 8 != block )
		{
			block = empty >> 8;
			base = block << 8;
			while ( SIInf_Base( h, base ) )
				base++;
		}

		// Setting for empty element.
		SIDic_WLeaf( h->dic, empty, 0 );
		SIDic_WBase( h->dic, empty, 0 );
		SIDic_WCheck( h->dic, empty, ( base ^ empty ) & 0xFF );
	}
}


// Build dictionary.


// Edit.
static	int		SIDic_Edit( hSIDic h, hSISort sort )
{
	int		ret;
	hSIInf	inf;

	if ( ( inf = SIInf_Create( h, sort ) ) == NULL )
		return	SI_EMEM;


	// Build dictionary.
	if ( ( ret = SIInf_ResizeTail( inf, sort->total + sort->n ) ) == SI_OK &&
		 ( ret = SIInf_Root( inf ) ) == SI_OK &&
		 ( ret = SIInf_BuildBC( inf ) ) == SI_OK &&
		 ( ret = SIInf_ReduceBC( inf ) ) == SI_OK )
	{
		SIInf_Revise( inf );
		SIInf_ReduceTail( inf );
	}

	SIInf_Delete( inf );

	return	ret;
}

// Build.
static	int		SIDic_Construct( hSIDic h, hSISet set )
{
	int		ret;
	hSISort		sort;

	// Sort key set.
	if ( ( ret = SISort_Open( &sort, set ) ) != SI_OK )
		return	ret;

	// Build dictionary.
	h->nkey = set->n;
	h->maxlen = sort->max;
	ret = SIDic_Edit( h, sort );

	// Free sorted key set.
	SISort_Close( sort );

	return	ret;
}


// Retrieval.


// Search key.
static	const char	*SIDic_ExactMatch( hSIDic h, const char *key, int *id )
{
	int		idx, depth = 0;
	char	*tail;

	// Follow arcs.
	idx = SIDic_Base( h, IDX_ROOT ) ^ ( unsigned char )key[depth];
	while ( !SIDic_Leaf( h, idx ) )
	{
		if ( SIDic_Check( h, idx ) != ( unsigned char )key[depth] )
			return	NULL;
		idx = SIDic_Base( h, idx ) ^ ( unsigned char )key[ ++depth ];
	}

	// Compare suffixes.
	tail = SIDic_Str( h, SIDic_Link( h, idx ) );
	if ( SIStr_Match( tail, key, &depth ) == NULL )
		return	NULL;

	if ( id != NULL )
		*id = idx;

	return	tail;
}

// Prefix match.
static	const char	*SIDic_PrefixMatch( hSIDic h, hSISrc src )
{
	int		leaf;
	const char	*tail = NULL;

	if ( src->idx == IDX_ROOT )
		src->len--;

	while ( !SIDic_Leaf( h, src->idx ) )
	{
		// Check arc with end marker.
		if ( src->start[ ++src->len ] != '\0' )
		{
			leaf = SIDic_Base( h, src->idx ) ^ '\0';
			if ( SIDic_Leaf( h, leaf ) )
			{
				tail = SIDic_Str( h, SIDic_Link( h, leaf ) );
				if ( tail[ src->len ] != '\0' )
					tail = NULL;
				else
					src->id = leaf;
			}
		}

		// Follow arc.
		src->idx = SIDic_Base( h, src->idx ) ^
				   ( unsigned char )src->start[ src->len ];
		if ( !SIDic_Leaf( h, src->idx ) )
		{
			if ( SIDic_Check( h, src->idx ) !=
				 ( unsigned char )src->start[ src->len ] )
			{
				src->idx = -1;
				return	tail;
			}
		}

		// Prefix.
		if ( tail != NULL )
			return	tail;
	}

	// Compare suffixes.
	tail = SIDic_Str( h, SIDic_Link( h, src->idx ) );
	tail = SIStr_Prefix( tail, src->start, &src->len );

	src->id = src->idx;
	src->idx = -1;

	return	tail;
}

// Infix match.
static	const char	*SIDic_InfixMatch( hSIDic h, hSISrc src )
{
	const char	*tail;

	while ( src->idx == -1 || ( tail = SIDic_PrefixMatch( h, src ) ) == NULL )
	{
		// No input remain.
		if ( src->start[0] == '\0' )
			return	NULL;

		// Slide.
		src->start++;
		src->len = 0;
		src->idx = IDX_ROOT;
	}

	return	tail;
}


// Dictionary header.


// Check header.
static	int		SIHead_Check( hSIHead h )
{
	// Check identifier.
	if ( memcmp( h->str, HEAD_STR, sizeof( h->str ) ) )
		return	SI_EFMT;

	// Check version.
	if ( h->version < SI_COMPAT )
		return	SI_EVER;

	return	SI_OK;
}

// Read header.
static	int		SIHead_Read( hSIHead h, FILE *fp )
{
	if ( fread( h, sizeof( *h ), 1, fp ) != 1 )
		return	SI_EIO;

	return	SIHead_Check( h );
}

// Write header.
static	int		SIHead_Write( hSIHead h, FILE *fp )
{
	if ( fwrite( h, sizeof( *h ), 1, fp ) != 1 )
		return	SI_EIO;

	return	SI_OK;
}


// Dictionary I/O.


// Restore dictionary from header.
static	int		SIDic_Restore( hSIDic h, hSIHead head )
{
	int		ret;

	h->nkey = head->nkey;
	h->maxlen = head->maxlen;

	if ( ( ret = SIDic_ResizeBC( h, head->nbc ) ) != SI_OK ||
		 ( ret = SIDic_ResizeTail( h, head->ntail ) ) != SI_OK )
		return	ret;

	return	SI_OK;
}

// Generate header.
static	void	SIDic_Head( hSIDic h, hSIHead head )
{
	memset( head, 0, sizeof( *head ) );

	// Set identifier and version.
	memcpy( head->str, HEAD_STR, sizeof( head->str ) );
	head->version = SI_VERSION;

	// Copy.
	head->nbc = h->nbc;
	head->ntail = h->ntail;
	head->nkey = h->nkey;
	head->maxlen = h->maxlen;
}

// Read dictionary.
static	int		SIDic_Read( hSIDic h, FILE *fp )
{
	int		ret;
	SIHead	head;

	// Read header.
	if ( ( ret = SIHead_Read( &head, fp ) ) != SI_OK )
		return	ret;
	if ( ( ret = SIDic_Restore( h, &head ) ) != SI_OK )
		return	ret;

	// Read body.
	if ( fread( h->bc, sizeof( *h->bc ) * h->nbc, 1, fp ) != 1 )
		return	SI_EIO;
	else if ( h->ntail && fread( h->tail, h->ntail, 1, fp ) != 1 )
		return	SI_EIO;

	return	SI_OK;
}

// Load dictionary.
static	int		SIDic_Load( hSIDic h, const char *path )
{
	int		ret;
	FILE	*fp;

	if ( ( fp = fopen( path, "rb" ) ) == NULL )
		return	SI_EIO;

	// Read dictionary.
	ret = SIDic_Read( h, fp );

	fclose( fp );

	return	ret;
}

// Output dictionary.
static	int		SIDic_Write( hSIDic h, FILE *fp )
{
	int		ret;
	SIHead	head;

	// Write header.
	SIDic_Head( h, &head );
	if ( ( ret = SIHead_Write( &head, fp ) ) != SI_OK )
		return	ret;

	// Write body.
	if ( fwrite( h->bc, sizeof( *h->bc ) * h->nbc, 1, fp ) != 1 )
		return	SI_EIO;
	else if ( h->ntail && fwrite( h->tail, h->ntail, 1, fp ) != 1 )
		return	SI_EIO;

	return	SI_OK;
}


// Interfaces.


// Build dictionary.
int		SIDic_Build( hSIDic *h, hSISet set )
{
	int		ret;

	if ( h == NULL )
		return	SI_EHDL;
	*h = NULL;
	if ( set == NULL )
		return	SI_EARG;

	if ( ( *h = SIDic_Create() ) == NULL )
		return	SI_EMEM;

	if ( ( ret = SIDic_Construct( *h, set ) ) != SI_OK )
	{
		SIDic_Delete( *h );
		*h = NULL;
	}

	return	ret;
}

// Open dictionary.
int		SIDic_Open( hSIDic *h, const char *path )
{
	int		ret;

	if ( h == NULL )
		return	SI_EHDL;
	*h = NULL;
	if ( path == NULL )
		return	SI_EARG;

	if ( ( *h = SIDic_Create() ) == NULL )
		return	SI_EHDL;

	if ( ( ret = SIDic_Load( *h, path ) ) != SI_OK )
	{
		SIDic_Delete( *h );
		*h = NULL;
	}

	return	ret;
}

// Destroy dictionary.
int		SIDic_Close( hSIDic h )
{
	if ( h == NULL )
		return	SI_EHDL;

	SIDic_Delete( h );

	return	SI_OK;
}

// Save dictionary.
int		SIDic_Save( hSIDic h, const char *path )
{
	int		ret;
	FILE	*fp;

	if ( h == NULL )
		return	SI_EHDL;
	else if ( path == NULL )
		return	SI_EARG;

	if ( ( fp = fopen( path, "wb" ) ) == NULL )
		return	SI_EIO;

	ret = SIDic_Write( h, fp );

	fclose( fp );

	return	ret;
}

// Get dictionary status.
int		SIDic_Status( hSIDic h, hSIStat st )
{
	if ( h == NULL )
		return	SI_EHDL;
	else if ( st == NULL )
		return	SI_EARG;

	// Copy.
	st->nbc = h->nbc;
	st->ntail = h->ntail;
	st->size = h->nbc * sizeof( *h->bc ) + h->ntail;
	st->nkey = h->nkey;
	st->maxlen = h->maxlen;

	return	SI_OK;
}

// String to ID.
const char	*SIDic_StrToID( hSIDic h, const char *s, int *id )
{
	if ( h == NULL || s == NULL )
		return	NULL;

	return	SIDic_ExactMatch( h, s, id );
}

// ID to string.
const char	*SIDic_IDToStr( hSIDic h, int id )
{
	if ( h == NULL )
		return	NULL;
	else if ( id < 0 || id >= h->nbc )
		return	NULL;
	else if ( !SIDic_Leaf( h, id ) )
		return	NULL;

	return	SIDic_Str( h, SIDic_Link( h, id ) );
}

// Start prefix and infix searches.
int		SISrc_Start( hSISrc h, const char *s )
{
	if ( h == NULL )
		return	SI_EHDL;
	else if ( s == NULL )
		return	SI_EARG;

	h->input = s;
	h->start = s;
	h->len = 0;
	h->idx = IDX_ROOT;

	return	SI_OK;
}

// Search keys which are prefixes of input.
const char	*SIDic_Prefix( hSIDic h, hSISrc src )
{
	if ( h == NULL || src == NULL )
		return	NULL;

	return	SIDic_PrefixMatch( h, src );
}

// Search keys in input.
const char	*SIDic_Infix( hSIDic h, hSISrc src )
{
	if ( h == NULL || src == NULL )
		return	NULL;

	return	SIDic_InfixMatch( h, src );
}


/*
	Error messages.
*/


// Return error message.
#define		SIDic_ErrorReturn( s )\
	{ static const char msg[] = s; return msg; }

// Error message.
const char	*SIDic_Error( int err )
{
	switch ( err )
	{
	case	SI_OK:		// Success.
		SIDic_ErrorReturn( "No error" );
	case	SI_EHDL:	// Invalid handle.
		SIDic_ErrorReturn( "Invalid handle" );
	case	SI_EMEM:	// Memory error.
		SIDic_ErrorReturn( "Memory allocation error" );
	case	SI_EIO:	// I/O error.
		SIDic_ErrorReturn( "I/O error" );
	case	SI_EARG:	// Invalid arguments.
		SIDic_ErrorReturn( "Invalid argument" );
	case	SI_EFMT:	// Invalid format.
		SIDic_ErrorReturn( "Invalid format" );
	case	SI_EMAX:	// Size limit.
		SIDic_ErrorReturn( "Too large" );
	case	SI_EVER:	// Version error.
		SIDic_ErrorReturn( "Version error" );
	case	SI_EKEY:	// No such key.
		SIDic_ErrorReturn( "Invalid key" );
	}

	SIDic_ErrorReturn( "Unknown error" );
}
