#ifndef		__STRING_ID_DOUBLE_ARRAY_H_
#define		__STRING_ID_DOUBLE_ARRAY_H_


// Functions return these integers.
#define		SI_OK		0		// No error.
#define		SI_EHDL		( -1 )	// Invalid handle.
#define		SI_EMEM		( -2 )	// Memory error.
#define		SI_EIO		( -3 )	// I/O error.
#define		SI_EARG		( -4 )	// Invalid arguments.
#define		SI_EFMT		( -5 )	// Invalid format.
#define		SI_EMAX		( -6 )	// Size limit.
#define		SI_EVER		( -7 )	// Version error.
#define		SI_EKEY		( -8 )	// No such key.


#ifdef	__cplusplus
extern	"C"	{
#endif


// Handle of key set.
typedef		struct SISet	*hSISet;

// Handle of dictionary.
typedef		struct SIDic	*hSIDic;

// Status of dictionary.
typedef		struct SIStat
{
	int		nbc;		// Length of BC.
	int		ntail;		// Length of TAIL.
	int		size;		// Number of bytes allocated to BC and TAIL.
	int		nkey;		// Number of keys.
	int		maxlen;		// Length of the longest key.
}	SIStat, *hSIStat;

// Agent for search.
typedef		struct SISrc
{
	const char	*input;		// Input.
	const char	*start;		// Start position of key.
	int			len;		// Length of key.
	int			idx;		// Index.
	int			id;			// ID.
}	SISrc, *hSISrc;


// Create key set.
int		SISet_Open( hSISet *h );
// Load key set.
int		SISet_Load( hSISet h, const char *path, const char *delim );
// Insert key.
int		SISet_Insert( hSISet h, const char *key );
// Destroy key set.
int		SISet_Close( hSISet h );

// Build dictionary.
int		SIDic_Build( hSIDic *h, hSISet set );
// Open dictionary.
int		SIDic_Open( hSIDic *h, const char *path );
// Destroy dictionary.
int		SIDic_Close( hSIDic h );
// Save dictionary.
int		SIDic_Save( hSIDic h, const char *path );

// Get dictionary status.
int		SIDic_Status( hSIDic h, hSIStat st );

// String to ID.
const char	*SIDic_StrToID( hSIDic h, const char *s, int *id );
// ID to string.
const char	*SIDic_IDToStr( hSIDic h, int id );

// Start prefix and infix searches.
int		SISrc_Start( hSISrc h, const char *s );
// Search keys which are prefixes of input.
const char	*SIDic_Prefix( hSIDic h, hSISrc src );
// Search keys in input.
const char	*SIDic_Infix( hSIDic h, hSISrc src );

// Error string.
const char	*SIDic_Error( int err );


#ifdef	__cplusplus
}
#endif


#endif
